#include "config.h"

const char* ConfigFilePath = "/config.json";
const char* ContactsFilePath = "/contacts.json";

void setup_storage(cfg_t *out) {
    if (!SPIFFS.begin(true)) {
        Serial.println("SPIFFS Mount Failed");
        return;
    }
    if (!SPIFFS.exists(ConfigFilePath)) {
        create_default_config();
    } else {
        read_config(out);
    }
    if (!SPIFFS.exists(ContactsFilePath)) {
        create_default_contactlist();
    }
}
void read_config(cfg_t *out) {
    File file = SPIFFS.open(ConfigFilePath, "r");
    if (!file) {
        Serial.println("Failed to open config file");
        return;
    }

    JsonDocument doc;
    DeserializationError error = deserializeJson(doc, file);
    if (error) {
        Serial.println("Failed to parse config file");
        return;
    }
    //
    // WiFi
    //
    String ap_ssid = doc[cfgKeyWiFiAPSSID].as<String>();
    String ap_pass = doc[cfgKeyWiFiAPPass].as<String>();
    String wifi1_ssid = doc[cfgKeyWiFi1SSID].as<String>();
    String wifi1_pass = doc[cfgKeyWiFi1Pass].as<String>();
    String wifi2_ssid = doc[cfgKeyWiFi2SSID].as<String>();
    String wifi2_pass = doc[cfgKeyWiFi2Pass].as<String>();
    wifi1_pass.toCharArray(out->wifi1_pass, sizeof(out->wifi1_pass));
    wifi1_ssid.toCharArray(out->wifi1_ssid, sizeof(out->wifi1_ssid));
    wifi2_pass.toCharArray(out->wifi2_pass, sizeof(out->wifi2_pass));
    wifi2_ssid.toCharArray(out->wifi2_ssid, sizeof(out->wifi2_ssid));
    ap_ssid.toCharArray(out->ap_ssid, sizeof(out->ap_ssid));
    ap_pass.toCharArray(out->ap_pass, sizeof(out->ap_pass));
    //
    // TX Modem
    //
    if (doc[cfgKeyTXFreq].is<float>()) out->tx_freq = doc[cfgKeyTXFreq].as<float>();
    if (doc[cfgKeyTXBaud].is<float>()) out->tx_baud = doc[cfgKeyTXBaud].as<float>();
    if (doc[cfgKeyTXTune].is<int>()) out->tx_tune = doc[cfgKeyTXTune].as<int>();
    if (doc[cfgKeyTXDeviation].is<float>()) out->tx_dev = doc[cfgKeyTXDeviation].as<float>();
    if (doc[cfgKeyTXPower].is<int>()) out->tx_power = doc[cfgKeyTXPower].as<int>();
    if (doc[cfgKeyTXFlagEmptyQueue].is<bool>()) out->tx_empty_queue = doc[cfgKeyTXFlagEmptyQueue].as<bool>();
    //
    // MQTT
    //
    String mqtt_host = doc[cfgKeyMQTTHost].as<String>();
    String mqtt_user = doc[cfgKeyMQTTUser].as<String>();
    String mqtt_pass = doc[cfgKeyMQTTPass].as<String>();
    String mqtt_topic = doc[cfgKeyMQTTTopic].as<String>();
    mqtt_host.toCharArray(out->mqtt_host, sizeof(out->mqtt_host));
    mqtt_user.toCharArray(out->mqtt_user, sizeof(out->mqtt_user));
    mqtt_pass.toCharArray(out->mqtt_pass, sizeof(out->mqtt_pass));
    mqtt_topic.toCharArray(out->mqtt_topic, sizeof(out->mqtt_topic));
    out->mqtt_port = doc[cfgKeyMQTTPort].as<int>();
    //
    // Broadcasts
    //
    // int broadcast_ric = doc["broadcast_ric"].as<int>();
    // int broadcast_fun = doc["broadcast_fun"].as<int>();
    if (doc[cfgKeyBroadcastRIC].is<int>()) out->broadcast_ric = doc[cfgKeyBroadcastRIC].as<int>();
    // out->broadcast_fun = broadcast_fun;


    //
    // DWD
    //
    if (doc[cfgKeyDWDEnable].is<bool>()) out->dwd_enable = doc[cfgKeyDWDEnable].as<bool>();
    if (doc[cfgKeyDWDInterval].is<int>()) out->dwd_interval = doc[cfgKeyDWDInterval].as<int>();
    if (doc[cfgKeyDWDFunction].is<int>()) out->dwd_fun = doc[cfgKeyDWDFunction].as<int>();
    if (doc[cfgKeyDWDRegion].is<String>()) {
        String dwd_region = doc[cfgKeyDWDRegion].as<String>();
        dwd_region.toCharArray(out->dwd_region, sizeof(out->dwd_region));
    }
    //
    // MoWaS
    //
    if (doc[cfgKeyMoWaSEnable].is<bool>()) out->mowas_enable = doc[cfgKeyMoWaSEnable].as<bool>();
    if (doc[cfgKeyMoWaSInterval].is<int>()) out->mowas_interval = doc[cfgKeyMoWaSInterval].as<int>();
    if (doc[cfgKeyMoWaSFunction].is<int>()) out->mowas_fun = doc[cfgKeyMoWaSFunction].as<int>();
    if (doc[cfgKeyMoWaSRegion].is<String>()) {
        String mowas_region = doc[cfgKeyMoWaSRegion].as<String>();
        mowas_region.toCharArray(out->mowas_region, sizeof(out->mowas_region));
    }
    //
    // DWD-WX
    //
    if (doc[cfgKeyDWDWXEnable].is<bool>()) out->dwdwx_enable = doc[cfgKeyDWDWXEnable].as<bool>();;
    if (doc[cfgKeyDWDWXFunction].is<int>()) out->dwdwx_fun = doc[cfgKeyDWDWXFunction].as<int>();
    if (doc[cfgKeyDWDWXLatitude].is<float>()) out->dwdwx_lat = doc[cfgKeyDWDWXLatitude].as<float>();
    if (doc[cfgKeyDWDWXLongitude].is<float>()) out->dwdwx_lng = doc[cfgKeyDWDWXLongitude].as<float>();
    //
    //
    if (doc[cfgKeyTimeInterval].is<int>()) {
        out->time_enable = doc[cfgKeyTimeEnable].as<bool>();
        out->time_interval = doc[cfgKeyTimeInterval].as<int>();
        out->time_ric = doc[cfgKeyTimeRIC].as<int>();
        out->time_fun = doc[cfgKeyTimeFunction].as<int>();
        out->time_mode = doc[cfgKeyTimeMode].as<int>();
        if (doc[cfgKeyTimeZone].is<String>()) {
            String time_zone = doc[cfgKeyTimeZone].as<String>();
            time_zone.toCharArray(out->time_zone, sizeof(out->time_zone));
        }
    }
    //
    if (doc[cfgKeyIdleInterval].is<int>()) out->idle_interval = doc[cfgKeyIdleInterval].as<int>();
    if (doc[cfgKeyIdleEnable].is<bool>()) out->idle_enable = doc[cfgKeyIdleEnable].as<bool>();
    if (doc[cfgKeyIdleMode].is<int>()) out->idle_mode = doc[cfgKeyIdleMode].as<int>();
    //
    if (doc[cfgKeyFlagPOCSAG_German].is<bool>()) out->pocsag_german = doc[cfgKeyFlagPOCSAG_German].as<bool>();
    if (doc[cfgKeyDeviceName].is<String>()) {
        String device_name = doc[cfgKeyDeviceName].as<String>();
        device_name.toCharArray(out->device_name, sizeof(out->device_name));
    }
    if (doc[cfgKeyOLEDTimeout].is<int>()) out->oled_timeout = doc[cfgKeyOLEDTimeout].as<int>();
    //
    if (doc[cfgKeyFlagWebserverProtect].is<bool>()) out->webserver_protect = doc[cfgKeyFlagWebserverProtect].as<bool>();
    
    // Serial.println("Loaded Configuration:");
    // Serial.printf("AP SSID: %s\n", out->ap_ssid);
    // Serial.printf("AP Password: %s\n", out->ap_pass);
    // Serial.printf("1. SSID: '%s'\n", out->wifi1_ssid);
    // Serial.printf("2. SSID: '%s'\n",out->wifi2_ssid);
    // Serial.printf("MQTT Host/Port: '%s':%d\n", out->mqtt_host, out->mqtt_port);
    // Serial.printf("Topic: '%s'\n",out->mqtt_topic);
    // Serial.println();
    // Serial.printf("German POCSAG: %d\n", out->pocsag_german);
    // Serial.printf("Transmitter: %f MHz @%dbd - dev=%dkHz\n", out->tx_freq, out->tx_baud, out->tx_dev);
    // Serial.printf("Broadcast: RIC %d\n", out->broadcast_ric);
    // Serial.printf("DWD: Enable=%d Interval=%dm SubRIC=%d Region=%s\n", out->dwd_enable, out->dwd_interval, out->dwd_fun, out->dwd_region);
    // Serial.printf("MoWaS: Enable=%d Interval=%dm SubRIC=%d Region=%s\n", out->mowas_enable, out->mowas_interval, out->mowas_fun, out->mowas_region);
    // Serial.printf("Time: Enable=%d Interval=%dm RIC=%d SubRIC=%d Mode=%d Zone=%s\n", out->time_enable, out->time_interval, out->time_ric, out->time_fun, out->time_mode, out->time_zone);
    //
    doc.clear();
    file.close();
}
String cfg_tostring() {
    File file = SPIFFS.open(ConfigFilePath, "r");
    if (!file) {
        Serial.println("Failed to open config file");
        return "";
    }
    String output = file.readString();
    file.close();
    return output;
}

void create_default_config() {
    // Serial.println("creating default config");

    JsonDocument doc;

    doc[cfgKeyWiFi1SSID] = cfgValEmpty;
    doc[cfgKeyWiFi1Pass] = cfgValEmpty;
    doc[cfgKeyWiFi2SSID] = cfgValEmpty;
    doc[cfgKeyWiFi2Pass] = cfgValEmpty;
    doc[cfgKeyWiFiAPSSID] = "POCSAG";
    doc[cfgKeyWiFiAPPass] = "Motorola";
    // modem
    doc[cfgKeyTXFreq] = 434.230f;
    doc[cfgKeyTXBaud] = 1200;
    doc[cfgKeyTXDeviation] = 4.5;
    doc[cfgKeyTXPower] = 13;
    doc[cfgKeyTXFlagEmptyQueue] = false;
    // mqtt
    doc[cfgKeyMQTTHost] = cfgValEmpty;
    doc[cfgKeyMQTTUser] = cfgValEmpty;
    doc[cfgKeyMQTTPass] = cfgValEmpty;
    doc[cfgKeyMQTTTopic] = cfgValEmpty;
    doc[cfgKeyMQTTPort] = 1883;
    //
    doc[cfgKeyBroadcastRIC] = 9999;
    //
    doc[cfgKeyDWDFunction] = 3;
    doc[cfgKeyDWDInterval] = 5;
    doc[cfgKeyDWDEnable] = false;
    doc[cfgKeyDWDRegion] = "";
    doc[cfgKeyMoWaSFunction] = 3;
    doc[cfgKeyMoWaSInterval] = 5;
    doc[cfgKeyMoWaSEnable] = false;
    doc[cfgKeyMoWaSRegion] = "";
    
    doc[cfgKeyTimeEnable] = false;
    doc[cfgKeyTimeInterval] = 720;
    doc[cfgKeyTimeRIC] = 0;
    doc[cfgKeyTimeFunction] = 3;
    doc[cfgKeyTimeMode] = 0;
    doc[cfgKeyTimeZone] = "Europe/Berlin";

    doc[cfgKeyDWDWXEnable] = false;
    doc[cfgKeyDWDWXFunction] = 3;
    doc[cfgKeyDWDWXLatitude] = 0.0;
    doc[cfgKeyDWDWXLongitude] = 0.0;

    doc[cfgKeyIdleEnable] = false;
    doc[cfgKeyIdleInterval] = 1;
    doc[cfgKeyIdleMode] = 0;

    doc[cfgKeyFlagPOCSAG_German] = false;
    doc[cfgKeyOLEDTimeout] = 2;
    doc[cfgKeyDeviceName] = "smartPOC";
    doc[cfgKeyFlagWebserverProtect] = false;
    File file = SPIFFS.open(ConfigFilePath, "w");
    if (!file) {
        Serial.println("Failed to open config file for writing");
        return;
    }

    serializeJson(doc, file);
    file.close();
    doc.clear();

    Serial.println("Configuration saved.");
}
//
void create_default_contactlist() {
    Serial.println("creating default contacts");

    JsonArray jArr;
    JsonObject testContact;
    testContact["r"] = 133701;
    testContact["n"] = "Contact Name";
    testContact["f"] = 3;
    testContact["tl"] = 80;
    testContact["t"] = "Test";
    jArr.add(testContact);
    //

    File file = SPIFFS.open(ContactsFilePath, "w");
    if (!file) {
        Serial.println("Failed to open config file for writing");
        return;
    }

    serializeJson(jArr, file);
    file.close();
    jArr.clear();

    Serial.println("Contacts saved.");
}
String contacts_tostring() {
    File file = SPIFFS.open(ContactsFilePath, "r");
    if (!file) {
        Serial.println("Failed to open contacts file");
        return "";
    }
    String output = file.readString();
    file.close();
    return output;
}
void contacts_write(String data) {
    File file = SPIFFS.open(ContactsFilePath, "w");
    if (!file) {
        Serial.println("Failed to open contacts file for writing");
        return;
    }
    file.print(data);
    file.close();
    Serial.println("Contacts saved.");
}
//
JsonDocument transactionJson;
void cfg_startTransaction() {
    File file = SPIFFS.open(ConfigFilePath, "r");
    if (!file) {
        Serial.println("Failed to open config file");
        return;
    }
    DeserializationError error = deserializeJson(transactionJson, file);
    if (error) {
        Serial.println("Failed to parse config file");
        return;
    }
    file.close();
}
void cfg_adjust(String key, bool val) {
    Serial.printf("adjustng cfg %s\n", key);
    transactionJson[key] = val;
}
void cfg_adjust(String key, String val) {
    Serial.printf("adjustng cfg %s = '%s'\n", key, val);
    transactionJson[key] = val;
}
void cfg_adjust(String key, float val) {
    Serial.printf("adjustng cfg %s = %f\n", key, val);
    transactionJson[key] = val;
}
void cfg_adjust(String key, int val) {
    Serial.printf("adjustng cfg %s = %d\n", key, val);
    transactionJson[key] = val;
}
void cfg_write() {
    File file = SPIFFS.open(ConfigFilePath, "w");
    if (!file) {
        Serial.println("Failed to open config file for writing");
        return;
    }
    serializeJson(transactionJson, file);
    file.close();
    //transactionJson.clear();
    Serial.println("wrote cfg");
}